@@ -64,6 +64,7 @@ static void
 z_ssl_locking_callback(int mode, int n, const char *file G_GNUC_UNUSED, int line G_GNUC_UNUSED)
 {
   z_enter();
+  
   if (n >= mutexnum)
     {
       /*LOG
@@ -173,7 +174,6 @@ void
 z_ssl_destroy(void)
 {
   ssl_initialized = 0;
-  g_free(ssl_mutexes);
 }
 
 static int
@@ -577,6 +577,13 @@ z_ssl_set_trusted_ca_list(SSL_CTX *ctx, gchar *ca_path)
 /* FIXME: SSL context cache */
 #ifndef G_OS_WIN32
 
+static int
+z_ssl_password(char *buf, int size, int rwflag, void *userdata)
+{
+  z_log(NULL, CORE_ERROR, 1, "Password protected key file detected;");
+  return -1;
+}
+
 ZSSLSession *
 z_ssl_session_new(char *session_id, 
                   int mode,
@@ -618,6 +625,7 @@ z_ssl_session_new(char *session_id,
 
   if (key_file && key_file[0])
     {
+      SSL_CTX_set_default_passwd_cb(ctx, z_ssl_password);
       ret = SSL_CTX_use_PrivateKey_file(ctx, key_file, SSL_FILETYPE_PEM);
       if (!ret)
         {
@@ -679,6 +687,13 @@ z_ssl_session_new(char *session_id,
               return NULL;
             }
         }
+      if (access(ca_dir, R_OK | X_OK) < 0)
+        {
+          z_log(session_id, CORE_ERROR, 3, "Insufficient permissions to CA directory; cadir='%s', error='%s'", ca_dir, g_strerror(errno));
+          SSL_CTX_free(ctx);
+          z_leave();
+          return NULL;
+        }
       if (!SSL_CTX_load_verify_locations(ctx, NULL, ca_dir))
         {
           /*LOG
@@ -692,7 +707,14 @@ z_ssl_session_new(char *session_id,
         }
       if (crl_dir && crl_dir[0])
         {
-          crl_store = z_ssl_crl_store_create(crl_dir);
+          if (access(crl_dir, R_OK | X_OK) < 0)
+            {
+              z_log(session_id, CORE_ERROR, 3, "Insufficient permissions to CRL directory; crldir='%s', error='%s'", crl_dir, g_strerror(errno));
+            }
+          else
+            {
+              crl_store = z_ssl_crl_store_create(crl_dir);
+            }
         }
       else
         {
