@@ -353,7 +353,7 @@ z_stream_line_get_copy(ZStream *s, gchar *line, gsize *length, GError **error)
 		whole line. It is likely caused by some max size limit or by some
 		standard default.
 	       */
-              z_log(NULL, CORE_ERROR, 2, "Line buffer too small; buffer='%.*s'", len, b);
+              z_log(NULL, CORE_ERROR, 2, "Line buffer too small; buffer='%.*s'", (gint) len, b);
               z_leave();
               return G_IO_STATUS_ERROR;
             }
